@@ -5,11 +5,12 @@ __email__ = 'hwaipy@gmail.com'
 import zmq
 from zmq.eventloop.zmqstream import ZMQStream
 from interactionfreepy.core import IFDefinition, IFException, Invocation, Message, IFLoop
-from tornado import websocket, web
+from tornado import websocket, web, httpserver
 from tornado.ioloop import IOLoop
 import time
 import logging
-
+import os
+import ssl
 
 class IFBroker(object):
     def __init__(self, binding, manager=None):
@@ -30,20 +31,24 @@ class IFBroker(object):
         self.main_stream.close()
         self.main_stream = None
 
-    def startWebSocket(self, port, path):
+    def startWebSocket(self, port, path, ssl_options=None):
         handlers_array = [
             (path, WebSocketZMQBridgeHandler),
+            (r"/(.+)", web.StaticFileHandler, {'path': 'interactionfreepy'}),
         ]
-        # settings = {
-        #     # 'debug': True,
-        # }
         app = web.Application(handlers_array)
-        app.listen(port)
+        if ssl_options:
+            ssl_ctx = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
+            ssl_ctx.load_cert_chain(ssl_options['certfile'], ssl_options['keyfile'])
+            httpserver.HTTPServer(app, ssl_options=ssl_ctx).listen(port)
+        else:
+            app.listen(port)
 
     def __onMessage(self, msg):
         try:
             sourcePoint, msg = msg[0], msg[1:]
             protocol = msg[1]
+            self.manager.statistics(sourcePoint, True, msg)
             if protocol != IFDefinition.PROTOCOL: raise IFException('Protocol {} not supported.'.format(protocol))
             distributingMode = msg[3]
             distributingAddress = msg[4]
@@ -91,6 +96,7 @@ class IFBroker(object):
             self.__sendMessage([sourcePoint] + errorMsg)
 
     def __sendMessage(self, frames):
+        self.manager.statistics(frames[0], False, frames)
         self.main_stream.send_multipart(frames)
 
 
@@ -102,12 +108,12 @@ class Manager:
         self.__activities = {}
         IOLoop.current().call_later(2, self.__check)
 
-    def registerAsService(self, sourcePoint, name, interfaces=[]):
-        if self.__services.__contains__(name):
+    def registerAsService(self, sourcePoint, name, interfaces=[], force=False):
+        if self.__services.__contains__(name) and not force:
             raise IFException('Service name [{}] occupied.'.format(name))
         if self.__workers.__contains__(sourcePoint):
             raise IFException('The current worker has registered as [{}].'.format(name))
-        self.__services[name] = [sourcePoint, interfaces, time.time()]
+        self.__services[name] = [sourcePoint, interfaces, time.time(), [0] * 4]
         self.__workers[sourcePoint] = name
         logging.info('Service [{}] registered as {}.'.format(name, interfaces))
 
@@ -137,9 +143,42 @@ class Manager:
         currentTime = time.time()
         for s in self.__services:
             meta = self.__services[s]
-            results.append([s, meta[0], meta[1], currentTime - meta[2]])
+            results.append({
+                "ServiceName": s, 
+                "Address": meta[0], 
+                "Interfaces": meta[1], 
+                "OnTime": currentTime - meta[2],
+                "Statistics": {
+                    "Received Message": meta[3][0],
+                    "Received Bytes": meta[3][1],
+                    "Sent Message": meta[3][2],
+                    "Sent Bytes": meta[3][3],
+                }
+            })
         return results
 
+    def statistics(self, sourcePoint, isReceived, message):
+        if self.__workers.__contains__(sourcePoint):
+            meta = self.__services[self.__workers[sourcePoint]]
+            offset = 0 if isReceived else 2
+            meta[3][offset] += 1
+            meta[3][offset + 1] += len(message[-1])
+            # print(sourcePoint, isReceived, (message[-1]), meta)
+
+    # async def stopService(self, sourcePoint, serviceName):
+    #     try:
+    #         print(serviceName)
+    #         # self.broker.main_stream.socket.disconnect(self.__services[serviceName][0]) 
+    #         print('stoping')
+    #         broker.
+    #         # import asyncio
+    #         # await asyncio.sleep(2)
+    #         # print('stoped')
+    #     except BaseException as e:
+    #         print('exception')
+    #         import traceback
+    #         traceback.print_tb(e)
+
     def __check(self):
         currentTime = time.time()
         for key in self.__activities.keys():
@@ -180,7 +219,11 @@ class WebSocketZMQBridgeHandler(websocket.WebSocketHandler):
 
 if __name__ == '__main__':
     broker = IFBroker("tcp://*:224")
-    broker.startWebSocket(81, '/ws/')
+    broker.startWebSocket(81, '/ws/', 
+    {
+        "certfile": "/root/.config/OpenSSL/server.crt",
+        "keyfile": "/root/.config/OpenSSL/server.key",
+    }
+    )
     print('started')
-
     IFLoop.join()
